@@ -57,17 +57,17 @@
 #include <time.h>
 
 #include "src/common/assoc_mgr.h"
-#include "src/common/gres.h"
+#include "src/interfaces/gres.h"
 #include "src/common/hostlist.h"
 #include "src/common/macros.h"
 #include "src/common/pack.h"
 #include "src/common/parse_time.h"
 #include "src/common/read_config.h"
-#include "src/common/select.h"
-#include "src/common/slurm_accounting_storage.h"
-#include "src/common/slurm_acct_gather_energy.h"
-#include "src/common/slurm_ext_sensors.h"
-#include "src/common/slurm_topology.h"
+#include "src/interfaces/select.h"
+#include "src/interfaces/accounting_storage.h"
+#include "src/interfaces/acct_gather_energy.h"
+#include "src/interfaces/ext_sensors.h"
+#include "src/interfaces/topology.h"
 #include "src/common/xassert.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
@@ -78,6 +78,8 @@ strong_alias(init_node_conf, slurm_init_node_conf);
 strong_alias(build_all_nodeline_info, slurm_build_all_nodeline_info);
 strong_alias(rehash_node, slurm_rehash_node);
 strong_alias(hostlist2bitmap, slurm_hostlist2bitmap);
+strong_alias(bitmap2node_name, slurm_bitmap2node_name);
+strong_alias(find_node_record, slurm_find_node_record);
 
 /* Global variables */
 List config_list  = NULL;	/* list of config_record entries */
@@ -86,10 +88,14 @@ time_t last_node_update = (time_t) 0;	/* time of last update */
 node_record_t **node_record_table_ptr = NULL;	/* node records */
 xhash_t* node_hash_table = NULL;
 int node_record_table_size = 0;		/* size of node_record_table_ptr */
-int node_record_count = 0;		/* count in node_record_table_ptr */
+int node_record_count = 0;		/* number of node slots in
+					 * node_record_table_ptr */
+int active_node_record_count = 0;	/* non-null node count in
+					 * node_record_table_ptr */
 int last_node_index = -1;		/* index of last node in tabe */
 uint16_t *cr_node_num_cores = NULL;
 uint32_t *cr_node_cores_offset = NULL;
+bool spec_cores_first = false;
 
 /* Local function definitions */
 static void _delete_config_record(void);
@@ -122,12 +128,11 @@ static void _delete_config_record(void)
  */
 static void xhash_walk_helper_cbk (void* item, void* arg)
 {
-	static int i = 0; /* sequential walk, so just update a static i */
-	int inx;
+	int *i_ptr = arg;
 	node_record_t *node_ptr = (node_record_t *) item;
 
-	inx = node_ptr -  node_record_table_ptr;
-	debug3("node_hash[%d]:%d(%s)", i++, inx, node_ptr->name);
+	debug3("node_hash[%d]:%d(%s)", (*i_ptr)++, node_ptr->index,
+	       node_ptr->name);
 }
 /*
  * _dump_hash - print the node_hash_table contents, used for debugging
@@ -137,11 +142,12 @@ static void xhash_walk_helper_cbk (void* item, void* arg)
  */
 static void _dump_hash (void)
 {
+	int i = 0;
 	if (node_hash_table == NULL)
 		return;
-	debug2("node_hash: indexing %ld elements",
-	      xhash_count(node_hash_table));
-	xhash_walk(node_hash_table, xhash_walk_helper_cbk, NULL);
+	debug2("node_hash: indexing %u elements",
+	       xhash_count(node_hash_table));
+	xhash_walk(node_hash_table, xhash_walk_helper_cbk, &i);
 }
 #endif
 
@@ -182,26 +188,17 @@ static void _node_record_hash_identity (void* item, const char** key,
  * globals: node_record_table_ptr - pointer to node table
  * NOTE: the caller must xfree the memory at node_list when no longer required
  */
-hostlist_t bitmap2hostlist (bitstr_t *bitmap)
+hostlist_t *bitmap2hostlist(bitstr_t *bitmap)
 {
-	int i, first, last;
-	hostlist_t hl;
+	hostlist_t *hl;
+	node_record_t *node_ptr;
 
 	if (bitmap == NULL)
 		return NULL;
 
-	first = bit_ffs(bitmap);
-	if (first == -1)
-		return NULL;
-
-	last  = bit_fls(bitmap);
 	hl = hostlist_create(NULL);
-	for (i = first; i <= last; i++) {
-		if (bit_test(bitmap, i) == 0)
-			continue;
-		if (!node_record_table_ptr[i])
-			continue;
-		hostlist_push_host(hl, node_record_table_ptr[i]->name);
+	for (int i = 0; (node_ptr = next_node_bitmap(bitmap, &i)); i++) {
+		hostlist_push_host(hl, node_ptr->name);
 	}
 	return hl;
 
@@ -219,7 +216,7 @@ hostlist_t bitmap2hostlist (bitstr_t *bitmap)
  */
 char * bitmap2node_name_sortable (bitstr_t *bitmap, bool sort)
 {
-	hostlist_t hl;
+	hostlist_t *hl;
 	char *buf;
 
 	hl = bitmap2hostlist (bitmap);
@@ -278,7 +275,7 @@ extern void build_all_frontend_info (bool is_slurmd_context)
 		fatal("No FrontendName information available!");
 
 	for (i = 0; i < count; i++) {
-		hostlist_t hl_name, hl_addr;
+		hostlist_t *hl_name, *hl_addr;
 		char *fe_name, *fe_addr;
 
 		fe_line = ptr_array[i];
@@ -334,18 +331,21 @@ extern void build_all_frontend_info (bool is_slurmd_context)
 #endif
 }
 
-static void _check_callback(char *alias, char *hostname,
-			    char *address, char *bcast_address,
-			    uint16_t port, int state_val,
-			    slurm_conf_node_t *node_ptr,
-			    config_record_t *config_ptr)
+static int _check_callback(char *alias, char *hostname,
+			   char *address, char *bcast_address,
+			   uint16_t port, int state_val,
+			   slurm_conf_node_t *node_ptr,
+			   config_record_t *config_ptr)
 {
+	int rc = SLURM_SUCCESS;
 	node_record_t *node_rec;
 
 	if ((node_rec = find_node_record2(alias)))
 		fatal("Duplicated NodeHostName %s in config file", alias);
 
-	node_rec = create_node_record(config_ptr, alias);
+	if ((rc = create_node_record(config_ptr, alias, &node_rec)))
+		return rc;
+
 	if ((state_val != NO_VAL) &&
 	    (state_val != NODE_STATE_UNKNOWN))
 		node_rec->node_state = state_val;
@@ -357,6 +357,8 @@ static void _check_callback(char *alias, char *hostname,
 	node_rec->port      = port;
 	node_rec->features  = xstrdup(node_ptr->feature);
 	node_rec->reason    = xstrdup(node_ptr->reason);
+
+	return rc;
 }
 
 extern config_record_t *config_record_from_conf_node(
@@ -418,7 +420,7 @@ extern void build_all_nodeline_info(bool set_bitmap, int tres_cnt)
 	for (i = 0; i < count; i++) {
 		node = ptr_array[i];
 		config_ptr = config_record_from_conf_node(node, tres_cnt);
-		expand_nodeline_info(node, config_ptr, _check_callback);
+		expand_nodeline_info(node, config_ptr, NULL, _check_callback);
 	}
 
 	if (set_bitmap) {
@@ -432,29 +434,129 @@ extern void build_all_nodeline_info(bool set_bitmap, int tres_cnt)
 	}
 }
 
+extern int build_node_spec_bitmap(node_record_t *node_ptr)
+{
+	uint32_t size;
+	int *cpu_spec_array;
+	int i;
+
+	if (node_ptr->tpc == 0) {
+		error("Node %s has invalid thread per core count (%u)",
+		      node_ptr->name, node_ptr->tpc);
+		return SLURM_ERROR;
+	}
+
+	if (!node_ptr->cpu_spec_list)
+		return SLURM_SUCCESS;
+	size = node_ptr->tot_cores;
+	FREE_NULL_BITMAP(node_ptr->node_spec_bitmap);
+	node_ptr->node_spec_bitmap = bit_alloc(size);
+	bit_set_all(node_ptr->node_spec_bitmap);
+
+	/* remove node's specialized cpus now */
+	cpu_spec_array = bitfmt2int(node_ptr->cpu_spec_list);
+	i = 0;
+	while (cpu_spec_array[i] != -1) {
+		int start = (cpu_spec_array[i] / node_ptr->tpc);
+		int end = (cpu_spec_array[i + 1] / node_ptr->tpc);
+		if (start > size) {
+			error("%s: Specialized CPUs id start above the configured limit.",
+			      __func__);
+			break;
+		}
+
+		if (end > size) {
+			error("%s: Specialized CPUs id end above the configured limit",
+			      __func__);
+			end = size;
+		}
+		/*
+		 * We need to test to make sure we have these bits in this map.
+		 * If the node goes from 12 cpus to 6 like scenario.
+		 */
+		bit_nclear(node_ptr->node_spec_bitmap, start, end);
+		i += 2;
+	}
+	node_ptr->core_spec_cnt = bit_clear_count(node_ptr->node_spec_bitmap);
+	xfree(cpu_spec_array);
+	return SLURM_SUCCESS;
+}
+
+/*
+ * Select cores and CPUs to be reserved for core specialization.
+ */
+static void _select_spec_cores(node_record_t *node_ptr)
+{
+	int spec_cores, res_core, res_sock, res_off;
+	int from_core, to_core, incr_core, from_sock, to_sock, incr_sock;
+	bitstr_t *cpu_spec_bitmap;
+
+	spec_cores = node_ptr->core_spec_cnt;
+
+	cpu_spec_bitmap = bit_alloc(node_ptr->cpus);
+	node_ptr->node_spec_bitmap = bit_alloc(node_ptr->tot_cores);
+	bit_set_all(node_ptr->node_spec_bitmap);
+
+	if (spec_cores_first) {
+		from_core = 0;
+		to_core   = node_ptr->cores;
+		incr_core = 1;
+		from_sock = 0;
+		to_sock   = node_ptr->tot_sockets;
+		incr_sock = 1;
+	} else {
+		from_core = node_ptr->cores - 1;
+		to_core   = -1;
+		incr_core = -1;
+		from_sock = node_ptr->tot_sockets - 1;
+		to_sock   = -1;
+		incr_sock = -1;
+	}
+	for (res_core = from_core;
+	     (spec_cores && (res_core != to_core)); res_core += incr_core) {
+		for (res_sock = from_sock;
+		     (spec_cores && (res_sock != to_sock));
+		      res_sock += incr_sock) {
+			int thread_off;
+			thread_off = ((res_sock * node_ptr->cores) + res_core) *
+				      node_ptr->tpc;
+			bit_nset(cpu_spec_bitmap, thread_off,
+				 thread_off + node_ptr->tpc - 1);
+			res_off = (res_sock * node_ptr->cores) + res_core;
+			bit_clear(node_ptr->node_spec_bitmap, res_off);
+			spec_cores--;
+		}
+	}
+
+	node_ptr->cpu_spec_list = bit_fmt_full(cpu_spec_bitmap);
+	FREE_NULL_BITMAP(cpu_spec_bitmap);
+
+	return;
+}
 /*
  * Expand a nodeline's node names, host names, addrs, ports into separate nodes.
  */
-extern void expand_nodeline_info(slurm_conf_node_t *node_ptr, config_record_t
-				 *config_ptr,
-				 void (*_callback) (
-				       char *alias, char *hostname,
-				       char *address, char *bcast_address,
-				       uint16_t port, int state_val,
-				       slurm_conf_node_t *node_ptr,
-				       config_record_t *config_ptr))
+extern int expand_nodeline_info(slurm_conf_node_t *node_ptr,
+				config_record_t *config_ptr,
+				char **err_msg,
+				int (*_callback) (
+					char *alias, char *hostname,
+					char *address, char *bcast_address,
+					uint16_t port, int state_val,
+					slurm_conf_node_t *node_ptr,
+					config_record_t *config_ptr))
 {
-	hostlist_t address_list = NULL;
-	hostlist_t alias_list = NULL;
-	hostlist_t bcast_list = NULL;
-	hostlist_t hostname_list = NULL;
-	hostlist_t port_list = NULL;
+	hostlist_t *address_list = NULL;
+	hostlist_t *alias_list = NULL;
+	hostlist_t *bcast_list = NULL;
+	hostlist_t *hostname_list = NULL;
+	hostlist_t *port_list = NULL;
 	char *address = NULL;
 	char *alias = NULL;
 	char *bcast_address = NULL;
 	char *hostname = NULL;
 	char *port_str = NULL;
-	int state_val = NODE_STATE_UNKNOWN;
+	int state_val = NODE_STATE_UNKNOWN, rc = SLURM_SUCCESS;
 	int address_count, alias_count, bcast_count, hostname_count, port_count;
 	uint16_t port = slurm_conf.slurmd_port;
 
@@ -563,8 +665,17 @@ extern void expand_nodeline_info(slurm_conf_node_t *node_ptr, config_record_t
 			port = port_int;
 		}
 
-		(*_callback)(alias, hostname, address, bcast_address,
-			     port, state_val, node_ptr, config_ptr);
+		if ((rc = (*_callback)(alias, hostname, address, bcast_address,
+				       port, state_val, node_ptr,
+				       config_ptr))) {
+			if (err_msg) {
+				xfree(*err_msg);
+				*err_msg = xstrdup_printf("%s (%s)",
+							  slurm_strerror(rc),
+							  alias);
+			}
+			break;
+		}
 
 		free(alias);
 	}
@@ -577,16 +688,13 @@ extern void expand_nodeline_info(slurm_conf_node_t *node_ptr, config_record_t
 		free(hostname);
 	if (port_str)
 		free(port_str);
-	if (address_list)
-		hostlist_destroy(address_list);
-	if (alias_list)
-		hostlist_destroy(alias_list);
-	if (bcast_list)
-		hostlist_destroy(bcast_list);
-	if (hostname_list)
-		hostlist_destroy(hostname_list);
-	if (port_list)
-		hostlist_destroy(port_list);
+	FREE_NULL_HOSTLIST(address_list);
+	FREE_NULL_HOSTLIST(alias_list);
+	FREE_NULL_HOSTLIST(bcast_list);
+	FREE_NULL_HOSTLIST(hostname_list);
+	FREE_NULL_HOSTLIST(port_list);
+
+	return rc;
 }
 
 /*
@@ -634,10 +742,9 @@ extern config_record_t *create_config_record(void)
 /*
  * Convert CPU list to reserve whole cores
  * OUT:
- *	node_ptr->node_spec_bitmap
  *	node_ptr->cpu_spec_list
  */
-static int _convert_cpu_spec_list(node_record_t *node_ptr, uint32_t tot_cores)
+static int _convert_cpu_spec_list(node_record_t *node_ptr)
 {
 	int i;
 	bitstr_t *cpu_spec_bitmap;
@@ -645,26 +752,13 @@ static int _convert_cpu_spec_list(node_record_t *node_ptr, uint32_t tot_cores)
 	/* create CPU bitmap from input CPU list */
 	cpu_spec_bitmap = bit_alloc(node_ptr->cpus);
 
-	if (bit_unfmt(cpu_spec_bitmap, node_ptr->cpu_spec_list)) {
-		error("CpuSpecList is invalid");
-	}
-
-	node_ptr->node_spec_bitmap = bit_alloc(tot_cores);
-
-	/* Create core spec bitmap from CPU bitmap */
-	for (i = 0; i < node_ptr->cpus; i++) {
-		if (bit_test(cpu_spec_bitmap, i))
-			bit_set(node_ptr->node_spec_bitmap,
-				(i / (node_ptr->tpc)));
-	}
-
 	/* Expand CPU bitmap to reserve whole cores */
-	for (i = 0; i < tot_cores; i++) {
-		if (bit_test(node_ptr->node_spec_bitmap, i)) {
+	for (i = 0; i < node_ptr->tot_cores; i++) {
+		if (!bit_test(node_ptr->node_spec_bitmap, i)) {
 			/* typecast to int to avoid coverity error */
 			bit_nset(cpu_spec_bitmap,
 				 (i * (int) node_ptr->tpc),
-				 ((i+1) * (int) node_ptr->tpc) - 1);
+				 ((i + 1) * (int) node_ptr->tpc) - 1);
 		}
 	}
 	xfree(node_ptr->cpu_spec_list);
@@ -683,7 +777,7 @@ static void _init_node_record(node_record_t *node_ptr,
 	 * registers.
 	 */
 	node_ptr->magic = NODE_MAGIC;
-	node_ptr->cpu_load = NO_VAL;
+	node_ptr->cpu_load = 0;
 	node_ptr->energy = acct_gather_energy_alloc(1);
 	node_ptr->ext_sensors = ext_sensors_alloc();
 	node_ptr->free_mem = NO_VAL64;
@@ -692,7 +786,8 @@ static void _init_node_record(node_record_t *node_ptr,
 	node_ptr->port = slurm_conf.slurmd_port;
 	node_ptr->protocol_version = SLURM_MIN_PROTOCOL_VERSION;
 	node_ptr->resume_timeout = NO_VAL16;
-	node_ptr->select_nodeinfo = select_g_select_nodeinfo_alloc();
+	if (running_in_slurmctld())
+		node_ptr->select_nodeinfo = select_g_select_nodeinfo_alloc();
 	node_ptr->suspend_time = NO_VAL;
 	node_ptr->suspend_timeout = NO_VAL16;
 
@@ -720,19 +815,11 @@ static void _init_node_record(node_record_t *node_ptr,
 
 	node_ptr->cpu_spec_list = xstrdup(config_ptr->cpu_spec_list);
 	if (node_ptr->cpu_spec_list) {
-		if (node_ptr->tpc > 1) {
-			_convert_cpu_spec_list(node_ptr, node_ptr->tot_cores);
-		} else {
-			node_ptr->node_spec_bitmap = bit_alloc(node_ptr->cpus);
-			if (bit_unfmt(node_ptr->node_spec_bitmap,
-				      node_ptr->cpu_spec_list)) {
-				error("CpuSpecList is invalid");
-			}
-		}
-		node_ptr->core_spec_cnt = bit_set_count(
-			node_ptr->node_spec_bitmap);
-		/* node_spec_bitmap is not set on spec cores. */
-		bit_not(node_ptr->node_spec_bitmap);
+		build_node_spec_bitmap(node_ptr);
+		if (node_ptr->tpc > 1)
+			_convert_cpu_spec_list(node_ptr);
+	} else if (node_ptr->core_spec_cnt) {
+		_select_spec_cores(node_ptr);
 	}
 
 	node_ptr->cpus_efctv = node_ptr->cpus -
@@ -741,8 +828,11 @@ static void _init_node_record(node_record_t *node_ptr,
 
 extern void grow_node_record_table_ptr(void)
 {
-	node_record_table_size = MAX(node_record_count + 100,
-				     slurm_conf.max_node_cnt);
+	node_record_table_size = node_record_count + 100;
+	if (slurm_conf.max_node_cnt != NO_VAL)
+		node_record_table_size = MAX(node_record_count,
+					     slurm_conf.max_node_cnt);
+
 	xrealloc(node_record_table_ptr,
 		 node_record_table_size * sizeof(node_record_t *));
 	/*
@@ -756,25 +846,27 @@ extern void grow_node_record_table_ptr(void)
  * create_node_record - create a node record and set its values to defaults
  * IN config_ptr - pointer to node's configuration information
  * IN node_name - name of the node
- * RET pointer to the record or NULL if error
+ * OUT node_ptr - node_record_t** with created node on SUCESS, NULL otherwise.
+ * RET SUCESS, or error code
  * NOTE: allocates memory at node_record_table_ptr that must be xfreed when
  *	the global node table is no longer required
  */
-extern node_record_t *create_node_record(config_record_t *config_ptr,
-					 char *node_name)
+extern int create_node_record(config_record_t *config_ptr, char *node_name,
+			      node_record_t **node_ptr)
 {
-	node_record_t *node_ptr;
 	xassert(config_ptr);
 	xassert(node_name);
+	xassert(node_ptr);
 
 	if (node_record_count >= node_record_table_size)
 		grow_node_record_table_ptr();
 
-	node_ptr = create_node_record_at(node_record_count, node_name,
-					 config_ptr);
+	if (!(*node_ptr = create_node_record_at(node_record_count,
+						node_name, config_ptr)))
+		return ESLURM_NODE_TABLE_FULL;
 	node_record_count++;
 
-	return node_ptr;
+	return SLURM_SUCCESS;
 }
 
 extern node_record_t *create_node_record_at(int index, char *node_name,
@@ -786,7 +878,8 @@ extern node_record_t *create_node_record_at(int index, char *node_name,
 	xassert(index <= node_record_count);
 	xassert(!node_record_table_ptr[index]);
 
-	if (slurm_conf.max_node_cnt && (index >= slurm_conf.max_node_cnt)) {
+	if ((slurm_conf.max_node_cnt != NO_VAL) &&
+	    (index >= slurm_conf.max_node_cnt)) {
 		error("Attempting to create node record past MaxNodeCount:%d",
 		      slurm_conf.max_node_cnt);
 		return NULL;
@@ -799,39 +892,47 @@ extern node_record_t *create_node_record_at(int index, char *node_name,
 	node_ptr->index = index;
 	node_ptr->name = xstrdup(node_name);
 	xhash_add(node_hash_table, node_ptr);
+	active_node_record_count++;
 
 	_init_node_record(node_ptr, config_ptr);
 
 	return node_ptr;
 }
 
-extern node_record_t *add_node_record(char *alias, config_record_t *config_ptr)
+extern int add_node_record(char *alias, config_record_t *config_ptr,
+			   node_record_t **node_ptr)
 {
-	node_record_t *node_ptr = NULL;
+	int rc = SLURM_SUCCESS;
 
-	if ((node_ptr = find_node_record2(alias))) {
-		error("Node '%s' already exists in the node table", alias);
-		return NULL;
+	xassert(node_ptr);
+
+	if ((*node_ptr = find_node_record2(alias))) {
+		rc = ESLURM_NODE_ALREADY_EXISTS;
+		goto end;
 	}
 
 	for (int i = 0; i < node_record_count; i++) {
 		if (node_record_table_ptr[i])
 			continue;
 
-		if (!(node_ptr = create_node_record_at(i, alias, config_ptr)))
-			return NULL;
+		if (!(*node_ptr =
+			create_node_record_at(i, alias, config_ptr))) {
+			rc = ESLURM_NODE_TABLE_FULL;
+			goto end;
+		}
 
 		bit_set(config_ptr->node_bitmap, i);
 
-		gres_init_node_config(node_ptr->config_ptr->gres,
-				      &node_ptr->gres_list);
+		gres_init_node_config((*node_ptr)->config_ptr->gres,
+				      &(*node_ptr)->gres_list);
 
 		break;
 	}
-	if (!node_ptr)
-		error("Unable to add node '%s', node table is full", alias);
+	if (!(*node_ptr))
+		rc = ESLURM_NODE_TABLE_FULL;
 
-	return node_ptr;
+end:
+	return rc;
 }
 
 static int _find_config_ptr(void *x, void *arg)
@@ -865,11 +966,11 @@ extern void insert_node_record(node_record_t *node_ptr)
 		node_ptr->index = i;
 		bit_set(node_ptr->config_ptr->node_bitmap, node_ptr->index);
 		xhash_add(node_hash_table, node_ptr);
+		active_node_record_count++;
 
 		/* re-add node to conf node hash tables */
-		slurm_reset_alias(node_ptr->name,
-				  node_ptr->comm_name,
-				  node_ptr->node_hostname);
+		slurm_conf_add_node(node_ptr);
+
 		return;
 	}
 
@@ -894,6 +995,7 @@ extern void delete_node_record(node_record_t *node_ptr)
 		if (i < 0)
 			last_node_index = -1;
 	}
+	active_node_record_count--;
 
 	_delete_node_config_ptr(node_ptr);
 
@@ -961,7 +1063,7 @@ static node_record_t *_find_node_record(char *name, bool test_alias,
 		return node_ptr;
 	}
 
-	if ((node_record_count == 1) &&
+	if ((node_record_count == 1) && node_record_table_ptr[0] &&
 	    (xstrcmp(node_record_table_ptr[0]->name, "localhost") == 0))
 		return (node_record_table_ptr[0]);
 
@@ -1013,6 +1115,10 @@ extern void init_node_conf(void)
 		config_list    = list_create (_list_delete_config);
 		front_end_list = list_create (destroy_frontend);
 	}
+	if (xstrcasestr(slurm_conf.sched_params, "spec_cores_first"))
+		spec_cores_first = true;
+	else
+		spec_cores_first = false;
 }
 
 
@@ -1036,7 +1142,13 @@ extern void node_fini2 (void)
 	}
 
 	xfree(node_record_table_ptr);
-	node_record_count = 0;
+	/*
+	 * Don't clear node_record_count because other plugins are relying on
+	 * node_record_count to free arrays on cleanup -- e.g.
+	 * free_core_array().
+	 *
+	 * node_record_count = 0;
+	 */
 }
 
 extern int node_name_get_inx(char *node_name)
@@ -1067,7 +1179,7 @@ extern int node_name2bitmap (char *node_names, bool best_effort,
 	int rc = SLURM_SUCCESS;
 	char *this_node_name;
 	bitstr_t *my_bitmap;
-	hostlist_t host_list;
+	hostlist_t *host_list;
 
 	my_bitmap = (bitstr_t *) bit_alloc (node_record_count);
 	*bitmap = my_bitmap;
@@ -1110,12 +1222,12 @@ extern int node_name2bitmap (char *node_names, bool best_effort,
  * OUT bitmap     - set to bitmap, may not have all bits set on error
  * RET 0 if no error, otherwise EINVAL
  */
-extern int hostlist2bitmap (hostlist_t hl, bool best_effort, bitstr_t **bitmap)
+extern int hostlist2bitmap(hostlist_t *hl, bool best_effort, bitstr_t **bitmap)
 {
 	int rc = SLURM_SUCCESS;
 	bitstr_t *my_bitmap;
 	char *name;
-	hostlist_iterator_t hi;
+	hostlist_iterator_t *hi;
 
 	FREE_NULL_BITMAP(*bitmap);
 	my_bitmap = (bitstr_t *) bit_alloc (node_record_count);
@@ -1176,10 +1288,14 @@ extern void purge_node_rec(node_record_t *node_ptr)
 	xfree(node_ptr->comm_name);
 	xfree(node_ptr->cpu_spec_list);
 	xfree(node_ptr->extra);
+	FREE_NULL_DATA(node_ptr->extra_data);
 	xfree(node_ptr->features);
 	xfree(node_ptr->features_act);
 	xfree(node_ptr->gres);
 	FREE_NULL_LIST(node_ptr->gres_list);
+	xfree(node_ptr->instance_id);
+	xfree(node_ptr->instance_type);
+	xfree(node_ptr->mcs_label);
 	xfree(node_ptr->name);
 	xfree(node_ptr->node_hostname);
 	FREE_NULL_BITMAP(node_ptr->node_spec_bitmap);
@@ -1187,10 +1303,12 @@ extern void purge_node_rec(node_record_t *node_ptr)
 	xfree(node_ptr->part_pptr);
 	xfree(node_ptr->power);
 	xfree(node_ptr->reason);
+	xfree(node_ptr->resv_name);
 	xfree(node_ptr->version);
 	acct_gather_energy_destroy(node_ptr->energy);
 	ext_sensors_destroy(node_ptr->ext_sensors);
-	select_g_select_nodeinfo_free(node_ptr->select_nodeinfo);
+	if (running_in_slurmctld())
+		select_g_select_nodeinfo_free(node_ptr->select_nodeinfo);
 	xfree(node_ptr->tres_str);
 	xfree(node_ptr->tres_fmt_str);
 	xfree(node_ptr->tres_cnt);
@@ -1308,26 +1426,6 @@ extern uint32_t cr_get_coremap_offset(uint32_t node_index)
 	return cr_node_cores_offset[node_record_count];
 }
 
-/* Return a bitmap the size of the machine in cores. On a Bluegene
- * system it will return a bitmap in cnodes. */
-extern bitstr_t *cr_create_cluster_core_bitmap(int core_mult)
-{
-	/* DEF_TIMERS; */
-	/* START_TIMER; */
-	bitstr_t *core_bitmap;
-	static int cnt = 0;
-
-	if (!cnt) {
-		cnt = cr_get_coremap_offset(node_record_count);
-		if (core_mult)
-			cnt *= core_mult;
-	}
-	core_bitmap = bit_alloc(cnt);
-	/* END_TIMER; */
-	/* info("creating of core bitmap of %d took %s", cnt, TIME_STR); */
-	return core_bitmap;
-}
-
 /*
  * Determine maximum number of CPUs on this node usable by a job
  * ntasks_per_core IN - tasks-per-core to be launched by this job
@@ -1354,7 +1452,7 @@ extern int adjust_cpus_nppcu(uint16_t ntasks_per_core, int cpus_per_task,
 
 extern char *find_hostname(uint32_t pos, char *hosts)
 {
-	hostlist_t hostlist = NULL;
+	hostlist_t *hostlist = NULL;
 	char *temp = NULL, *host = NULL;
 
 	if (!hosts || (pos == NO_VAL) || (pos == INFINITE))
@@ -1374,7 +1472,8 @@ extern node_record_t *next_node(int *index)
 {
 	xassert(index);
 
-	if (*index >= node_record_count)
+	if (!node_record_table_ptr ||
+	    (*index >= node_record_count))
 		return NULL;
 
 	while (!node_record_table_ptr[*index]) {
@@ -1390,6 +1489,31 @@ extern node_record_t *next_node(int *index)
 	return node_record_table_ptr[*index];
 }
 
+extern node_record_t *next_node_bitmap(bitstr_t *bitmap, int *index)
+{
+	xassert(index);
+
+	if (!node_record_table_ptr ||
+	    (*index >= node_record_count))
+		return NULL;
+
+	xassert(bitmap);
+	xassert(bit_size(bitmap) == node_record_count);
+
+	while (true) {
+		*index = bit_ffs_from_bit(bitmap, *index);
+		if (*index == -1)
+			return NULL;
+		if (node_record_table_ptr[*index])
+			break;
+		(*index)++;  /* Skip blank entries */
+	}
+
+	xassert(node_record_table_ptr[*index]->index == *index);
+
+	return node_record_table_ptr[*index];
+}
+
 extern bitstr_t *node_conf_get_active_bitmap(void)
 {
 	bitstr_t *b = bit_alloc(node_record_count);
@@ -1402,3 +1526,36 @@ extern void node_conf_set_all_active_bits(bitstr_t *b)
 	for (int i = 0; next_node(&i); i++)
 		bit_set(b, i);
 }
+
+extern char *node_conf_nodestr_tokenize(char *s, char **save_ptr)
+{
+	char *end;
+
+	if (s == NULL)
+		s = *save_ptr;
+
+	xassert(s); /* If s is NULL here we are using this function wrong */
+
+	if (*s == '\0') {
+		*save_ptr = s;
+		return NULL;
+	}
+
+	/* token ends with a comma not followed by a digit */
+	end = s;
+	while (*end && ((end[0] != ',') || isdigit(end[1])))
+		end++;
+
+	if (*end)
+		*end++ = '\0';
+	*save_ptr = end;
+	return s;
+}
+
+extern void node_conf_create_cluster_core_bitmap(bitstr_t **core_bitmap)
+{
+	if (*core_bitmap)
+		return;
+
+	*core_bitmap = bit_alloc(cr_get_coremap_offset(node_record_count));
+}
