@@ -1163,8 +1163,8 @@ static void _handle_include(char *include_file, char *conf_file)
  */
 static int _parse_include_directive(s_p_hashtbl_t *hashtbl, uint32_t *hash_val,
 				    const char *line, char **leftover,
-				    bool ignore_new, char *slurm_conf_path,
-				    char *last_ancestor, bool check_permissions)
+				    uint32_t flags, char *slurm_conf_path,
+				    char *last_ancestor)
 {
 	char *ptr;
 	char *fn_start, *fn_stop;
@@ -1194,7 +1194,7 @@ static int _parse_include_directive(s_p_hashtbl_t *hashtbl, uint32_t *hash_val,
 		path_name = get_extra_conf_path(file_name);
 
 		stat(path_name, &temp);
-		if ((check_permissions) &&
+		if ((flags & PARSE_FLAGS_CHECK_PERMISSIONS) &&
 		   ((temp.st_mode & (S_IRWXU | S_IRWXG | S_IRWXO)) != 0600))
 			fatal("Included file %s at %s should be 600 is %o accessible for group or others",
 			      file_name,
@@ -1202,8 +1202,8 @@ static int _parse_include_directive(s_p_hashtbl_t *hashtbl, uint32_t *hash_val,
 			      temp.st_mode & (S_IRWXU | S_IRWXG | S_IRWXO));
 		if (!last_ancestor)
 			last_ancestor = xbasename(slurm_conf_path);
-		rc = s_p_parse_file(hashtbl, hash_val, path_name, ignore_new,
-				    last_ancestor, check_permissions);
+		rc = s_p_parse_file(hashtbl, hash_val, path_name, flags,
+				    last_ancestor);
 		xfree(path_name);
 		if (rc == SLURM_SUCCESS) {
 			if (!xstrstr(file_name, "/") && running_in_slurmctld())
@@ -1220,7 +1220,7 @@ static int _parse_include_directive(s_p_hashtbl_t *hashtbl, uint32_t *hash_val,
 }
 
 int s_p_parse_file(s_p_hashtbl_t *hashtbl, uint32_t *hash_val, char *filename,
-		   bool ignore_new, char *last_ancestor, bool check_permissions)
+		   uint32_t flags, char *last_ancestor)
 {
 	FILE *f;
 	char *leftover = NULL;
@@ -1230,6 +1230,7 @@ int s_p_parse_file(s_p_hashtbl_t *hashtbl, uint32_t *hash_val, char *filename,
 	int inc_rc;
 	struct stat stat_buf;
 	char *line = NULL;
+	bool ignore_new = (flags & PARSE_FLAGS_IGNORE_NEW);
 
 	if (!filename) {
 		error("s_p_parse_file: No filename given.");
@@ -1271,10 +1272,9 @@ int s_p_parse_file(s_p_hashtbl_t *hashtbl, uint32_t *hash_val, char *filename,
 		}
 
 		inc_rc = _parse_include_directive(hashtbl, hash_val,
-						  line, &leftover, ignore_new,
-						  filename, last_ancestor,
-						  check_permissions);
-		if (inc_rc == 0) {
+						  line, &leftover, flags,
+						  filename, last_ancestor);
+		if (inc_rc == 0 && !(flags & PARSE_FLAGS_INCLUDE_ONLY)) {
 			if (!_parse_next_key(hashtbl, line, &leftover,
 					     ignore_new)) {
 				rc = SLURM_ERROR;
@@ -1625,7 +1625,7 @@ static int _parse_expline_doexpand(s_p_hashtbl_t** tables,
 				   int tables_count,
 				   s_p_values_t* item)
 {
-	hostlist_t item_hl, sub_item_hl;
+	hostlist_t *item_hl, *sub_item_hl;
 	int item_count, i;
 	int j, items_per_record, items_idx = 0;
 	char* item_str = NULL;
@@ -1665,7 +1665,7 @@ static int _parse_expline_doexpand(s_p_hashtbl_t** tables,
 	 * of key tables n (entities) and (m mod(n)) is zero, then split the
 	 * set of expanded values in n consecutive sets (strings).
 	 */
-	item_hl = (hostlist_t)(item->data);
+	item_hl = item->data;
 	item_count = hostlist_count(item_hl);
 	if ((item_count < tables_count) || (item_count == 1)) {
 		items_per_record = 1;
@@ -1747,7 +1747,7 @@ int s_p_parse_line_expanded(const s_p_hashtbl_t *hashtbl,
 	s_p_hashtbl_t* strtbl = NULL;
 	s_p_hashtbl_t** tables = NULL;
 	int tables_count = 0;
-	hostlist_t value_hl = NULL;
+	hostlist_t *value_hl = NULL;
 	char* value_str = NULL;
 	s_p_values_t* attr = NULL;
 
@@ -1813,8 +1813,7 @@ int s_p_parse_line_expanded(const s_p_hashtbl_t *hashtbl,
 cleanup:
 	if (value_str)
 		free(value_str);
-	if (value_hl)
-		hostlist_destroy(value_hl);
+	FREE_NULL_HOSTLIST(value_hl);
 	s_p_hashtbl_destroy(strtbl);
 
 	if (status == SLURM_ERROR && tables) {
