@@ -163,6 +163,16 @@ static const struct luaL_Reg slurm_functions [] = {
 	{ NULL, NULL }
 };
 
+static void _register_slurm_output_errtab(lua_State *L)
+{
+	int i;
+
+	for (i = 0; i < slurm_errtab_size; i++) {
+		lua_pushnumber(L, slurm_errtab[i].xe_number);
+		lua_setfield(L, -2, slurm_errtab[i].xe_name);
+	}
+}
+
 static void _register_slurm_output_functions(lua_State *L)
 {
 	char *unpack_str;
@@ -227,22 +237,7 @@ static void _register_slurm_output_functions(lua_State *L)
 	lua_setfield(L, -2, "FAILURE");
 	lua_pushnumber(L, SLURM_SUCCESS);
 	lua_setfield(L, -2, "SUCCESS");
-	lua_pushnumber(L, ESLURM_ACCESS_DENIED);
-	lua_setfield(L, -2, "ESLURM_ACCESS_DENIED");
-	lua_pushnumber(L, ESLURM_ACCOUNTING_POLICY);
-	lua_setfield(L, -2, "ESLURM_ACCOUNTING_POLICY");
-	lua_pushnumber(L, ESLURM_INVALID_ACCOUNT);
-	lua_setfield(L, -2, "ESLURM_INVALID_ACCOUNT");
-	lua_pushnumber(L, ESLURM_INVALID_LICENSES);
-	lua_setfield(L, -2, "ESLURM_INVALID_LICENSES");
-	lua_pushnumber(L, ESLURM_INVALID_NODE_COUNT);
-	lua_setfield(L, -2, "ESLURM_INVALID_NODE_COUNT");
-	lua_pushnumber(L, ESLURM_INVALID_TIME_LIMIT);
-	lua_setfield(L, -2, "ESLURM_INVALID_TIME_LIMIT");
-	lua_pushnumber(L, ESLURM_JOB_MISSING_SIZE_SPECIFICATION);
-	lua_setfield(L, -2, "ESLURM_JOB_MISSING_SIZE_SPECIFICATION");
-	lua_pushnumber(L, ESLURM_MISSING_TIME_LIMIT);
-	lua_setfield(L, -2, "ESLURM_MISSING_TIME_LIMIT");
+	_register_slurm_output_errtab(L);
 
 	/*
 	 * Other definitions needed to interpret data
@@ -254,7 +249,7 @@ static void _register_slurm_output_functions(lua_State *L)
 	lua_setfield(L, -2, "ALLOC_SID_USER_HOLD");
 	lua_pushnumber(L, INFINITE);
 	lua_setfield(L, -2, "INFINITE");
-	lua_pushnumber(L, INFINITE64);
+	lua_pushnumber(L, (double) INFINITE64);
 	lua_setfield(L, -2, "INFINITE64");
 	lua_pushnumber(L, MAIL_INVALID_DEPEND);
 	lua_setfield(L, -2, "MAIL_INVALID_DEPEND");
@@ -288,7 +283,7 @@ static void _register_slurm_output_functions(lua_State *L)
 	lua_setfield(L, -2, "JOB_SHARED_USER");
 	lua_pushnumber(L, JOB_SHARED_MCS);
 	lua_setfield(L, -2, "JOB_SHARED_MCS");
-	lua_pushnumber(L, NO_VAL64);
+	lua_pushnumber(L, (double) NO_VAL64);
 	lua_setfield(L, -2, "NO_VAL64");
 	lua_pushnumber(L, NO_VAL);
 	lua_setfield(L, -2, "NO_VAL");
@@ -306,6 +301,10 @@ static void _register_slurm_output_functions(lua_State *L)
 	lua_setfield(L, -2, "GRES_DISABLE_BIND");
 	lua_pushnumber(L, GRES_ENFORCE_BIND);
 	lua_setfield(L, -2, "GRES_ENFORCE_BIND");
+	lua_pushnumber(L, GRES_MULT_TASKS_PER_SHARING);
+	lua_setfield(L, -2, "GRES_MULT_TASKS_PER_SHARING");
+	lua_pushnumber(L, GRES_ONE_TASK_PER_SHARING);
+	lua_setfield(L, -2, "GRES_ONE_TASK_PER_SHARING");
 	lua_pushnumber(L, KILL_INV_DEP);
 	lua_setfield(L, -2, "KILL_INV_DEP");
 	lua_pushnumber(L, NO_KILL_INV_DEP);
@@ -390,6 +389,8 @@ extern int slurm_lua_job_record_field(lua_State *L, const job_record_t *job_ptr,
 		lua_pushnumber(L, job_ptr->end_time);
 	} else if (!xstrcmp(name, "exit_code")) {
 		lua_pushnumber(L, job_ptr->exit_code);
+	} else if (!xstrcmp(name, "extra")) {
+		lua_pushstring(L, job_ptr->extra);
 	} else if (!xstrcmp(name, "features")) {
 		if (job_ptr->details)
 			lua_pushstring(L, job_ptr->details->features);
@@ -420,6 +421,8 @@ extern int slurm_lua_job_record_field(lua_State *L, const job_record_t *job_ptr,
 			lua_pushnumber(L, job_ptr->details->max_nodes);
 		else
 			lua_pushnumber(L, 0);
+	} else if (!xstrcmp(name, "mcs_label")) {
+		lua_pushstring(L, job_ptr->mcs_label);
 	} else if (!xstrcmp(name, "mem_per_tres")) {
 		lua_pushstring(L, job_ptr->mem_per_tres);
 	} else if (!xstrcmp(name, "min_cpus")) {
@@ -483,7 +486,7 @@ extern int slurm_lua_job_record_field(lua_State *L, const job_record_t *job_ptr,
 		if (job_ptr->details)
 			lua_pushnumber(L, job_ptr->details->pn_min_memory);
 		else
-			lua_pushnumber(L, NO_VAL64);
+			lua_pushnumber(L, (double) NO_VAL64);
 	} else if (!xstrcmp(name, "priority")) {
 		lua_pushnumber(L, job_ptr->priority);
 	} else if (!xstrcmp(name, "qos")) {
@@ -513,7 +516,7 @@ extern int slurm_lua_job_record_field(lua_State *L, const job_record_t *job_ptr,
 				lua_pushnil(L);
 		} else
 			lua_pushnil(L);
-		free_buf(bscript);
+		FREE_NULL_BUFFER(bscript);
 	} else if (!xstrcmp(name, "selinux_context")) {
 		lua_pushstring(L, job_ptr->selinux_context);
  	} else if (!xstrcmp(name, "site_factor")) {
@@ -666,12 +669,12 @@ extern int slurm_lua_loadscript(lua_State **L, const char *plugin,
 
 	if (stat(script_path, &st) != 0) {
 		if (curr) {
-			(void) error("%s: Unable to stat %s, using old script: %s",
-			             plugin, script_path, strerror(errno));
+			error("%s: Unable to stat %s, using old script: %s",
+			      plugin, script_path, strerror(errno));
 			return SLURM_SUCCESS;
 		}
-		(void) error("%s: Unable to stat %s: %s",
-		             plugin, script_path, strerror(errno));
+		error("%s: Unable to stat %s: %s",
+		      plugin, script_path, strerror(errno));
 		return SLURM_ERROR;
 	}
 
@@ -682,9 +685,7 @@ extern int slurm_lua_loadscript(lua_State **L, const char *plugin,
 	}
 	debug3("%s: %s: loading Lua script: %s", __func__, plugin, script_path);
 
-	/*
-	 *  Initilize lua
-	 */
+	/* Initialize lua */
 	if (!(new = luaL_newstate())) {
 		error("%s: %s: luaL_newstate() failed to allocate.",
 		      plugin, __func__);
@@ -742,13 +743,13 @@ extern int slurm_lua_loadscript(lua_State **L, const char *plugin,
 	rc = (int) lua_tonumber(new, -1);
 	if (rc != SLURM_SUCCESS) {
 		if (curr) {
-			(void) error("%s: %s: returned %d on load, using previous script",
-			             plugin, script_path, rc);
+			error("%s: %s: returned %d on load, using previous script",
+			      plugin, script_path, rc);
 			lua_close(new);
 			return SLURM_SUCCESS;
 		}
-		(void) error("%s: %s: returned %d on load", plugin,
-			     script_path, rc);
+		error("%s: %s: returned %d on load", plugin,
+		      script_path, rc);
 		lua_pop(new, 1);
 		lua_close(new);
 		return SLURM_ERROR;
@@ -760,8 +761,8 @@ extern int slurm_lua_loadscript(lua_State **L, const char *plugin,
 	rc = _check_lua_script_functions(new, plugin, script_path, req_fxns);
 	if (rc != SLURM_SUCCESS) {
 		if (curr) {
-			(void) error("%s: %s: required function(s) not present, using previous script",
-			             plugin, script_path);
+			error("%s: %s: required function(s) not present, using previous script",
+			      plugin, script_path);
 			lua_close(new);
 			return SLURM_SUCCESS;
 		}
