@@ -88,7 +88,7 @@ static int    dbd_sigarray[] = {	/* blocked signals for this process */
 	SIGUSR2, SIGTSTP, SIGXCPU, SIGQUIT,
 	SIGPIPE, SIGALRM, SIGABRT, SIGHUP, 0 };
 static int    debug_level = 0;		/* incremented for -v on command line */
-static int    foreground = 0;		/* run process as a daemon */
+static bool daemonize = true;		/* run process as a daemon */
 static int    setwd = 0;		/* change working directory -s  */
 static log_options_t log_opts = 	/* Log to stderr & syslog */
 	LOG_OPTS_INITIALIZER;
@@ -108,7 +108,6 @@ static void  _become_slurm_user(void);
 static void  _commit_handler_cancel(void);
 static void *_commit_handler(void *no_data);
 static void  _daemonize(void);
-static void  _default_sigaction(int sig);
 static void  _init_config(void);
 static void  _init_pidfile(void);
 static void  _kill_old_slurmdbd(void);
@@ -142,7 +141,7 @@ int main(int argc, char **argv)
 	_update_nice();
 
 	_kill_old_slurmdbd();
-	if (foreground == 0)
+	if (daemonize)
 		_daemonize();
 
 	/*
@@ -156,21 +155,21 @@ int main(int argc, char **argv)
 
 	/*
 	 * Do plugin init's after _init_pidfile so systemd is happy as
-	 * slurm_acct_storage_init() could take a long time to finish if running
+	 * acct_storage_g_init() could take a long time to finish if running
 	 * for the first time after an upgrade.
 	 */
-	if (slurm_auth_init(NULL) != SLURM_SUCCESS) {
+	if (auth_g_init() != SLURM_SUCCESS) {
 		fatal("Unable to initialize authentication plugins");
 	}
 	if (hash_g_init() != SLURM_SUCCESS) {
 		fatal("failed to initialize hash plugin");
 	}
-	if (slurm_acct_storage_init() != SLURM_SUCCESS) {
+	if (acct_storage_g_init() != SLURM_SUCCESS) {
 		fatal("Unable to initialize %s accounting storage plugin",
 		      slurm_conf.accounting_storage_type);
 	}
 
-	if (foreground == 0 || setwd)
+	if (daemonize || setwd)
 		_set_work_dir();
 	log_config();
 	init_dbd_stats();
@@ -194,9 +193,10 @@ int main(int argc, char **argv)
 
 	/*
 	 * If we are tracking wckey we need to cache wckeys,
-	 * if we aren't only cache the users, qos, and tres.
+	 * if we aren't only cache the assoc, users, qos, and tres.
 	 */
 	assoc_init_arg.cache_level = ASSOC_MGR_CACHE_USER |
+		ASSOC_MGR_CACHE_ASSOC |
 		ASSOC_MGR_CACHE_QOS | ASSOC_MGR_CACHE_TRES;
 	if (slurmdbd_conf->track_wckey)
 		assoc_init_arg.cache_level |= ASSOC_MGR_CACHE_WCKEY;
@@ -322,8 +322,9 @@ end_it:
 	}
 
 	assoc_mgr_fini(0);
-	slurm_acct_storage_fini();
-	slurm_auth_fini();
+	acct_storage_g_fini();
+	auth_g_fini();
+	hash_g_fini();
 	log_fini();
 	free_slurmdbd_conf();
 	slurm_mutex_lock(&rpc_mutex);
@@ -470,7 +471,7 @@ static void _parse_commandline(int argc, char **argv)
 	while ((c = getopt(argc, argv, "Dhn:R::svV")) != -1)
 		switch (c) {
 		case 'D':
-			foreground = 1;
+			daemonize = 0;
 			break;
 		case 'h':
 			_usage(argv[0]);
@@ -546,14 +547,14 @@ static void _update_logging(bool startup)
 
 	log_opts.logfile_level = slurmdbd_conf->debug_level;
 
-	if (foreground)
+	if (!daemonize)
 		log_opts.stderr_level  = slurmdbd_conf->debug_level;
 	else
 		log_opts.stderr_level = LOG_LEVEL_QUIET;
 
 	if (slurmdbd_conf->syslog_debug != LOG_LEVEL_END) {
 		log_opts.syslog_level =	slurmdbd_conf->syslog_debug;
-	} else if (foreground) {
+	} else if (!daemonize) {
 		log_opts.syslog_level = LOG_LEVEL_QUIET;
 	} else if ((slurmdbd_conf->debug_level > LOG_LEVEL_QUIET)
 		   && !slurmdbd_conf->log_file) {
@@ -887,11 +888,11 @@ static void *_signal_handler(void *no_data)
 	(void) pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
 
 	/* Make sure no required signals are ignored (possibly inherited) */
-	_default_sigaction(SIGINT);
-	_default_sigaction(SIGTERM);
-	_default_sigaction(SIGHUP);
-	_default_sigaction(SIGABRT);
-	_default_sigaction(SIGUSR2);
+	xsignal_default(SIGINT);
+	xsignal_default(SIGTERM);
+	xsignal_default(SIGHUP);
+	xsignal_default(SIGABRT);
+	xsignal_default(SIGUSR2);
 
 	while (1) {
 		xsignal_sigset_create(sig_array, &set);
@@ -924,24 +925,6 @@ static void *_signal_handler(void *no_data)
 
 }
 
-/* Reset some signals to their default state to clear any
- * inherited signal states */
-static void _default_sigaction(int sig)
-{
-	struct sigaction act;
-
-	if (sigaction(sig, NULL, &act)) {
-		error("sigaction(%d): %m", sig);
-		return;
-	}
-	if (act.sa_handler != SIG_IGN)
-		return;
-
-	act.sa_handler = SIG_DFL;
-	if (sigaction(sig, &act, NULL))
-		error("sigaction(%d): %m", sig);
-}
-
 static void _become_slurm_user(void)
 {
 	gid_t slurm_user_gid;
